module Redify
  module Base

    extend ActiveSupport::Concern

    module ClassMethods

      # Call this from an ActiveRecord model to activate Redify features.
      def redify(options = {})
        
        # includes main Redify functionality
        __send__ :include, Redify::Model

        # Redis handler for the model, if specified
        __send__ :redify_handler=, options[:through]
        
        # caches some keys to avoid unnecessary calls to get_*_key_name_for() methods
        __send__ :redify_keys=, {
          :resource => Redify::Core.get_namespaced_key_name_for(:resource => self),
          :all => Redify::Core.get_namespaced_key_name_for(:resource => self, :collection => "All")
        }

        # load data into Redis for the first time
        Redify::Core.initialize_resource(self)

        # hooks
        after_create lambda { Redify::Core.after_create(self) }
        after_save lambda { Redify::Core.after_save(self) }
        after_destroy lambda { Redify::Core.after_destroy(self) }

      end

    end
  end
end
