module Redify
  module Model

    extend ActiveSupport::Concern

    module ClassMethods

      attr_accessor :redify_keys

      def redify_handler
        @redify_handler || Redify::Settings.default_handler
      end
      attr_writer :redify_handler

      def find (*args)

        if args.size > 0 && args.last.is_a?(Hash)
          # let ActiveRecord deal with queries with extended options, including :skip_redify
          args.last.delete(:skip_redify)
          return super(*args)
        end

        a = args.at(0)
        return send(a) if a == :first || a == :last || a == :all

        records = []
        
        args.each do |id|
          records << Redify::Core.fetch_record(:resource => self, :id => id)
        end

        args.size == 1 ? records.first : records

      end

      def first (*args)
        Redify::Core.fetch_records_by_edge(self, false, args.first)
      end

      def last (*args)
        Redify::Core.fetch_records_by_edge(self, true, args.first)
      end

      def all (*args)
        records = []
        redify_handler.lrange(redify_keys[:all], 0, -1).each do |record_id|
          records << Redify::Core.fetch_record(:resource => self, :id => record_id)
        end
        records
      end

    end

  end
end